/**
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package com.funambol.syncclient.spdm;

import java.util.Hashtable;

/**
 * This interface models a device management node. A management not can be either
 * a configuration context (a tree element that contains other nodes) or a
 * configuration node (which instead contains configuration properties).
 *
 *
 *
 * @version $Id: ManagementNode.java,v 1.4 2007/01/11 19:21:40 nichele Exp $
 */
public interface ManagementNode {

    /**
     * Returns the node's context
     *
     * @return the node's context
     */
    public String getContext();

    /**
     * Returns the entire node's context path (concatenation of all parent
     * context paths).
     *
     * @return the entire node's context path
     */
    public String getFullContext();

    /**
     * Returns the given config parameter
     *
     * @param name the configration parameter name
     * @return this node value
     *
     * @throws DMException in case of errors.
     */
    public Object getValue(String name)
    throws DMException;


    /**
     * @return hashtable of values
     *
     * @throws DMException in case of errors.
     */
    public Hashtable getValues()
    throws DMException;


    /**
     * Retrieves a value from the given management subnode name.
     *
     * @param node the subnode containing the config value specified by name
     * @param name the name of the configuration value to return
     *
     * @return the node value
     *
     * @throws DMException in case of errors.
     */
    public Object getNodeValue(String node, String name)
    throws DMException;

    /**
     * Retrieves all values in the given management subnode name.
     *
     * @param node the subnode containing the required config values
     *
     * @return the node values
     *
     * @throws DMException in case of errors.
     */
    public Hashtable getNodeValues(String node)
    throws DMException;

   /**
    * Reads the properties of the given node and creates an object with the
    * values read.
    * @param node the subnode containing the required config values
    * @return the object
    * @throws DMException in case of errors.
    */
    public Object getManagementObject(String node)
    throws DMException;

    /**
     * Retrieves the children subnodes of the current node.
     *
     * @return an array of <i>ManagementNode</i> containing this node's children
     *
     * @throws DMException in case of errors.
     */
    public ManagementNode[] getChildren()
    throws DMException;

    /**
     * Retrieves the subnode with the given name
     *
     * @return context the subnode context (name)
     *
     * @throws DMException in case of errors.
     */
    public ManagementNode getChildNode(String context)
    throws DMException;


    /**
     * Retrieves this node's parent.
     *
     * @return the node's parent
     *
     * @throws DMException in case of errors.
     */
    public ManagementNode getParent()
    throws DMException;


    /**
     * Sets the node value as a whole object.
     *
     * @param name the configuration parameter name
     * @param value the value
     *
     * @throws DMException in case of errors.
     */
    void setValue(String name, Object value) throws DMException;


    /**
     * Sets a subnode specific value.
     *
     * @param node subnode to set
     * @param name configuration parameter to set
     * @param value
     *
     * @throws DMException in case of errors.
     */
    void setValue(String node, String name, Object value) throws DMException;


    /**
     * Remove subnode from configuration three.
     *
     * @param node subnode to reomove
     *
     * @throws DMException in case of errors.
     */
    void removeNode(String node) throws DMException;


}
